mod types;

use axum::{
    extract::Json,
    response,
    routing::{get, post},
    Router,
};
use log::{debug, info};
use serde::Serialize;
use tokio::net::TcpListener;
use types::{
    simulation::Board,
    wire::{Request, Response},
    Direction,
};

#[tokio::main]
async fn main() {
    env_logger::init();

    debug!("Creating routes");
    let app = Router::new()
        .route("/", get(info))
        .route("/start", post(start))
        .route("/move", post(get_move))
        .route("/end", post(end));

    debug!("Creating listener");
    let listener = TcpListener::bind("0.0.0.0:8000").await.unwrap();
    debug!("Starting server");
    axum::serve(listener, app).await.unwrap();
}

async fn info() -> response::Json<Info> {
    info!("got info request");
    response::Json(Info {
        apiversion: "1",
        author: "der-informatiker",
        color: "#00FFEE",
        head: "smart-caterpillar",
        tail: "mouse",
        version: env!("CARGO_PKG_VERSION"),
    })
}

#[derive(Debug, Clone, Serialize)]
struct Info {
    apiversion: &'static str,
    author: &'static str,
    color: &'static str,
    head: &'static str,
    tail: &'static str,
    version: &'static str,
}

async fn start(request: Json<Request>) {
    let board = Board::from(&*request);
    info!("got start request: {board}");
}

async fn get_move(request: Json<Request>) -> response::Json<Response> {
    let board = Board::from(&*request);
    info!("got move request: {board}");
    response::Json(Response {
        direction: Direction::Up,
        shout: None,
    })
}

async fn end(request: Json<Request>) {
    let board = Board::from(&*request);
    info!("got end request: {board}");
}
